local Hero = require 'lib.hero'
local Sketchy = require 'lib.sketchy'

local lp = love.physics

love.physics.setMeter(1)
local viewer = Sketchy():setScale(70):setCamera(0.5, 0)

local drawShape = viewer.drawShape
function viewer:drawShape (shape, fixture)
    local data = fixture:getUserData()
    local color = data and data.color
    if color and color.enabled then
        love.graphics.setColor(color.r, color.g, color.b)
    end
    return drawShape(viewer, shape, fixture)
end
function viewer:presentBody () end
function viewer:drawJoint () end

local worldState = require 'world'
local world = Hero.load(worldState)

function getFixtureAtPoint (x, y)
    local result
    world:queryBoundingBox(
        x, y, x, y,
        function (fixture)
            if fixture:testPoint(x, y) then
                result = fixture
                return false
            end
            return true
        end)
    return result
end

function getBodyAtPoint (x, y)
    local fixture = getFixtureAtPoint(x, y)
    if fixture then
        return fixture:getBody()
    end
end

function destroyMouseJoint ()
    if mouseJoint and not mouseJoint:isDestroyed() then
        mouseJoint:destroy()
    end
    mouseJoint = nil
end

function createMouseJoint (x, y)
    local wx, wy = viewer:screenToWorld(x, y)
    local body = getBodyAtPoint(wx, wy)
    
    destroyMouseJoint()
    
    if body then
        mouseJoint = lp.newMouseJoint(body, wx, wy)
    end
end

function love.update (dt)
    world:update(dt)
end

function love.draw ()
    viewer:draw(world)
end

function love.mousepressed (x, y, button)
    createMouseJoint(x, y)
end

function love.mousereleased (x, y, button)
    destroyMouseJoint()
end

function love.mousemoved (x, y)
    if mouseJoint then
        mouseJoint:setTarget(viewer:screenToWorld(x, y))
    end
end

function love.keypressed ()
    destroyMouseJoint()
    for _, joint in ipairs(world:getJointList()) do
        if joint.setFrequency then
            joint:setFrequency(0)
        end
    end
end

function love.keyreleased ()
    for _, joint in ipairs(world:getJointList()) do
        if joint.setFrequency then
            joint:setFrequency(0.75)
        end
    end
end
